#include <stdio.h>
#include <math.h>

#include "filters.c"

static double testfilt(double x)
{
    if (x < 0.0)x = -x;
	return x*x*x;
}

static double (*const filterfun)(double) = Lanczos;
static const double Support = 3.0;

static void Analyze(double *data, int beginpos, int endpos)
{
	const int datasize = endpos-beginpos+1;
	const int destsize = datasize*2;
	
	const double factor = (double)datasize / destsize;
	double scalefactor = factor;
	if(scalefactor < 1.0)scalefactor = 1.0;
	double support = Support * scalefactor;
	if(support < 0.5)support = 0.5;
	
	double *weight = new double[(int)(support*2+3)];
	
	for(int d=0; d<destsize; ++d)
	{
		double center = d * factor;
		int start = (int)(0.5 + center - support);
		int end   = (int)(0.5 + center + support);
		
		double density = 0;
		int n=0;

		for(int i=start; i<end; ++i)
		{
			double distance = i - center + 0.5;
			//distance *= pow(2,distance/12.0);
			density += weight[n++] = filterfun(distance/scalefactor) / scalefactor;
		}
/**/
		if(density != 0)
			for(int i=0; i<n; ++i)
				weight[i] /= density;
/**/
		double value = 0;
		for(int i=0,p=start; i<n; ++i,++p)
			value += weight[i] * (data[p]);
		
		printf("%12.4f (%8.2f)\n", value, data[d*datasize/destsize]);
	}
	
	delete[] weight;
}

int main(void)
{
	// Two examples curves.
	
	double data1[] = {20.81, 23.24, 26.29, 30.21,
	                  35.42, 42.69, 53.51,   71.27, 105.72, 200.73,
	                  1632.48,
	                  274.94, 128.92, 85.20,   64.22, 51.93, 43.87,
	                  38.18, 33.95, 30.68, 28.07};
	
	Analyze(data1, 124,144);
	
	puts("---------------------------------");
	
	double data2[] = {53.97, 59.78, 66.99, 76.18,
	                  88.34, 105.15, 129.91,   169.92, 245.20, 438.24,
	                  1989.86,
	                  798.13, 335.81, 214.31,   158.41, 126.37, 105.64,
	                  91.17, 80.49, 72.30, 65.81};
	
	Analyze(data2, 258,278);
}
